import { CommonModule } from '@angular/common';
import { HttpClient, HttpParams, HttpRequest } from '@angular/common/http';
import { forOwn, isArray, mapValues, isBoolean } from 'lodash';
import { __awaiter } from 'tslib';
import { InjectionToken, Injectable, Inject, NgModule, Optional, SkipSelf } from '@angular/core';
import { Router } from '@angular/router';
import { JwtHelperService } from '@auth0/angular-jwt';
import { NgForage } from 'ngforage';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
const HttpMethods = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    PATCH: 'PUT',
    DELETE: 'DELETE',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const EnvironmentConfigService = new InjectionToken('EnvironmentConfig');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const methodsWithBody = [HttpMethods.POST, HttpMethods.PUT, HttpMethods.PATCH];
/** @type {?} */
const uploadingSection = ['media'];
class HttpService {
    /**
     * @param {?} config
     * @param {?} http
     */
    constructor(config, http) {
        this.config = config;
        this.http = http;
    }
    /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    handle(urlParams, method, options) {
        options = this.filterParams(urlParams, options);
        if (methodsWithBody.indexOf(method) === -1) {
            options = {
                params: new HttpParams({
                    fromObject: options
                })
            };
        }
        /** @type {?} */
        let request = new HttpRequest(method, this.buildUrl(urlParams), options, {
            responseType: urlParams.responseType || 'json'
        });
        // add header to prevent token checking for public requests
        if (urlParams.insecure === true) {
            request = request.clone({ setHeaders: { Insecure: '1' } });
            delete urlParams.insecure;
        }
        return this.http.request(request)
            .toPromise()
            .then((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            // console.log('response', event);
            return {
                result: true,
                data: (urlParams.rawResponse === true) ? event : event.body
            };
        }))
            .catch((/**
         * @param {?} error
         * @return {?}
         */
        (error) => {
            return {
                result: false,
                error: (urlParams.rawResponse === true) ? error : error.error
            };
        }));
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    get(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.GET, params);
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    search(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.GET, params);
    }
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    save(urlParams, data) {
        return this.handle(urlParams, HttpMethods.POST, data);
    }
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    update(urlParams, data) {
        return this.handle(urlParams, HttpMethods.PUT, data);
    }
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    delete(urlParams, params = {}) {
        return this.handle(urlParams, HttpMethods.DELETE, params);
    }
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    filterParams(urlParams, options) {
        /** @type {?} */
        let params = {};
        forOwn(options, (/**
         * @param {?} value
         * @param {?} key
         * @return {?}
         */
        (value, key) => {
            if (isArray(value) && key.indexOf('[]') === -1) {
                key += '[]';
            }
            if (value !== null && value !== undefined) {
                params[key] = value;
            }
        }));
        // convert boolean values to numeric except media content
        if (uploadingSection.indexOf(urlParams.section) === -1) {
            params = mapValues(params, (/**
             * @param {?} value
             * @return {?}
             */
            value => isBoolean(value) ? +value : value));
        }
        return params;
    }
    /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    buildUrl(urlParams) {
        /** @type {?} */
        const url = [`${this.config.url}/${urlParams.section}`];
        if (urlParams.id !== undefined) {
            url.push(`${urlParams.id}`);
        }
        if (urlParams.cmd) {
            url.push(`${urlParams.cmd}`);
        }
        if (urlParams.secondId) {
            url.push(`${urlParams.secondId}`);
        }
        return url.join('/');
    }
}
HttpService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
HttpService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [EnvironmentConfigService,] }] },
    { type: HttpClient }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AuthService {
    /**
     * @param {?} httpService
     * @param {?} router
     * @param {?} storageService
     */
    constructor(httpService, router, storageService) {
        this.httpService = httpService;
        this.router = router;
        this.storageService = storageService;
        this.jwtHelper = new JwtHelperService();
        this._user = null;
    }
    /**
     * @return {?}
     */
    getToken() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.storageService.getItem('jwt');
        });
    }
    /**
     * @param {?} token
     * @return {?}
     */
    setToken(token) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.storageService.setItem('jwt', token);
        });
    }
    /**
     * Set current logged user
     * @param {?} user
     * @return {?}
     */
    setUser(user) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.storageService.setItem('user', user);
        });
    }
    /**
     * @return {?}
     */
    getUser() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._user) {
                /** @type {?} */
                const user = yield this.storageService.getItem('user');
                this._user = user;
                return user;
            }
            else {
                return this._user;
            }
        });
    }
    /**
     * Check if current user is authenticated
     * @return {?}
     */
    isAuthenticated() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const token = yield this.getToken();
            return token !== null && !this.isExpired();
        });
    }
    /**
     * Check if token is expired
     * @return {?}
     */
    isExpired() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const token = yield this.getToken();
            return token !== null && this.jwtHelper.isTokenExpired(token);
        });
    }
    /**
     * Attempt to refresh token
     * @return {?}
     */
    refreshToken() {
        return this.httpService
            .search({
            section: 'auth',
            cmd: 'refresh',
            rawResponse: true,
            insecure: true
        })
            .then((/**
         * @param {?} response
         * @return {?}
         */
        (response) => __awaiter(this, void 0, void 0, function* () {
            if (response.result) {
                /** @type {?} */
                const newToken = response.data.headers.get('authorization').split(' ')[1];
                /** @type {?} */
                const claims = this.jwtHelper.decodeToken(newToken);
                yield this.setUser(claims.user);
                yield this.setToken(newToken);
                return newToken;
            }
            else {
                return '';
            }
        })));
    }
    /**
     * Clear all session variables
     * @return {?}
     */
    logout() {
        return __awaiter(this, void 0, void 0, function* () {
            this._user = null;
            yield this.storageService.removeItem('jwt');
            yield this.storageService.removeItem('user');
        });
    }
}
AuthService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthService.ctorParameters = () => [
    { type: HttpService },
    { type: Router },
    { type: NgForage }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ApiLibModule {
    /**
     * @param {?} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('ApiModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: ApiLibModule,
            providers: [
                { provide: EnvironmentConfigService, useValue: config },
                { provide: HttpService, useClass: 'ApiHttpService' },
                { provide: AuthService, useClass: 'ApiAuthService' },
            ]
        };
    }
}
ApiLibModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                imports: [
                    CommonModule
                ],
                exports: []
            },] }
];
/** @nocollapse */
ApiLibModule.ctorParameters = () => [
    { type: ApiLibModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiLibModule, AuthService as ɵc, EnvironmentConfigService as ɵa, HttpService as ɵb };

//# sourceMappingURL=api-lib.js.map