import { CommonModule } from '@angular/common';
import { HttpClient, HttpParams, HttpRequest } from '@angular/common/http';
import { forOwn, isArray, mapValues, isBoolean } from 'lodash';
import { __awaiter, __generator } from 'tslib';
import { InjectionToken, Injectable, Inject, NgModule, Optional, SkipSelf } from '@angular/core';
import { Router } from '@angular/router';
import { JwtHelperService } from '@auth0/angular-jwt';
import { NgForage } from 'ngforage';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {string} */
var HttpMethods = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    PATCH: 'PUT',
    DELETE: 'DELETE',
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var EnvironmentConfigService = new InjectionToken('EnvironmentConfig');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
var methodsWithBody = [HttpMethods.POST, HttpMethods.PUT, HttpMethods.PATCH];
/** @type {?} */
var uploadingSection = ['media'];
var HttpService = /** @class */ (function () {
    function HttpService(config, http) {
        this.config = config;
        this.http = http;
    }
    /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    HttpService.prototype.handle = /**
     * @private
     * @param {?} urlParams
     * @param {?} method
     * @param {?} options
     * @return {?}
     */
    function (urlParams, method, options) {
        options = this.filterParams(urlParams, options);
        if (methodsWithBody.indexOf(method) === -1) {
            options = {
                params: new HttpParams({
                    fromObject: options
                })
            };
        }
        /** @type {?} */
        var request = new HttpRequest(method, this.buildUrl(urlParams), options, {
            responseType: urlParams.responseType || 'json'
        });
        // add header to prevent token checking for public requests
        if (urlParams.insecure === true) {
            request = request.clone({ setHeaders: { Insecure: '1' } });
            delete urlParams.insecure;
        }
        return this.http.request(request)
            .toPromise()
            .then((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            // console.log('response', event);
            return {
                result: true,
                data: (urlParams.rawResponse === true) ? event : event.body
            };
        }))
            .catch((/**
         * @param {?} error
         * @return {?}
         */
        function (error) {
            return {
                result: false,
                error: (urlParams.rawResponse === true) ? error : error.error
            };
        }));
    };
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    HttpService.prototype.get = /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    function (urlParams, params) {
        if (params === void 0) { params = {}; }
        return this.handle(urlParams, HttpMethods.GET, params);
    };
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    HttpService.prototype.search = /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    function (urlParams, params) {
        if (params === void 0) { params = {}; }
        return this.handle(urlParams, HttpMethods.GET, params);
    };
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    HttpService.prototype.save = /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    function (urlParams, data) {
        return this.handle(urlParams, HttpMethods.POST, data);
    };
    /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    HttpService.prototype.update = /**
     * @param {?} urlParams
     * @param {?} data
     * @return {?}
     */
    function (urlParams, data) {
        return this.handle(urlParams, HttpMethods.PUT, data);
    };
    /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    HttpService.prototype.delete = /**
     * @param {?} urlParams
     * @param {?=} params
     * @return {?}
     */
    function (urlParams, params) {
        if (params === void 0) { params = {}; }
        return this.handle(urlParams, HttpMethods.DELETE, params);
    };
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @param urlParams
     * @param options
     */
    /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    HttpService.prototype.filterParams = /**
     * Transform keys to `key[]` for array type vales
     * and remove params with empty values
     * @private
     * @param {?} urlParams
     * @param {?} options
     * @return {?}
     */
    function (urlParams, options) {
        /** @type {?} */
        var params = {};
        forOwn(options, (/**
         * @param {?} value
         * @param {?} key
         * @return {?}
         */
        function (value, key) {
            if (isArray(value) && key.indexOf('[]') === -1) {
                key += '[]';
            }
            if (value !== null && value !== undefined) {
                params[key] = value;
            }
        }));
        // convert boolean values to numeric except media content
        if (uploadingSection.indexOf(urlParams.section) === -1) {
            params = mapValues(params, (/**
             * @param {?} value
             * @return {?}
             */
            function (value) { return isBoolean(value) ? +value : value; }));
        }
        return params;
    };
    /**
     * Make url like `/section/id/cmd/secondId`
     * @param urlParams
     */
    /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    HttpService.prototype.buildUrl = /**
     * Make url like `/section/id/cmd/secondId`
     * @private
     * @param {?} urlParams
     * @return {?}
     */
    function (urlParams) {
        /** @type {?} */
        var url = [this.config.url + "/" + urlParams.section];
        if (urlParams.id !== undefined) {
            url.push("" + urlParams.id);
        }
        if (urlParams.cmd) {
            url.push("" + urlParams.cmd);
        }
        if (urlParams.secondId) {
            url.push("" + urlParams.secondId);
        }
        return url.join('/');
    };
    HttpService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    HttpService.ctorParameters = function () { return [
        { type: undefined, decorators: [{ type: Inject, args: [EnvironmentConfigService,] }] },
        { type: HttpClient }
    ]; };
    return HttpService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var AuthService = /** @class */ (function () {
    function AuthService(httpService, router, storageService) {
        this.httpService = httpService;
        this.router = router;
        this.storageService = storageService;
        this.jwtHelper = new JwtHelperService();
        this._user = null;
    }
    /**
     * @return {?}
     */
    AuthService.prototype.getToken = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.storageService.getItem('jwt')];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * @param {?} token
     * @return {?}
     */
    AuthService.prototype.setToken = /**
     * @param {?} token
     * @return {?}
     */
    function (token) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.storageService.setItem('jwt', token)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * Set current logged user
     * @param {User} user
     */
    /**
     * Set current logged user
     * @param {?} user
     * @return {?}
     */
    AuthService.prototype.setUser = /**
     * Set current logged user
     * @param {?} user
     * @return {?}
     */
    function (user) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.storageService.setItem('user', user)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * @return {?}
     */
    AuthService.prototype.getUser = /**
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var user;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!!this._user) return [3 /*break*/, 2];
                        return [4 /*yield*/, this.storageService.getItem('user')];
                    case 1:
                        user = _a.sent();
                        this._user = user;
                        return [2 /*return*/, user];
                    case 2: return [2 /*return*/, this._user];
                }
            });
        });
    };
    /**
     * Check if current user is authenticated
     * @return {boolean}
     */
    /**
     * Check if current user is authenticated
     * @return {?}
     */
    AuthService.prototype.isAuthenticated = /**
     * Check if current user is authenticated
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var token;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getToken()];
                    case 1:
                        token = _a.sent();
                        return [2 /*return*/, token !== null && !this.isExpired()];
                }
            });
        });
    };
    /**
     * Check if token is expired
     * @return {boolean}
     */
    /**
     * Check if token is expired
     * @return {?}
     */
    AuthService.prototype.isExpired = /**
     * Check if token is expired
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            var token;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getToken()];
                    case 1:
                        token = _a.sent();
                        return [2 /*return*/, token !== null && this.jwtHelper.isTokenExpired(token)];
                }
            });
        });
    };
    /**
     * Attempt to refresh token
     * @return {Promise<string>}
     */
    /**
     * Attempt to refresh token
     * @return {?}
     */
    AuthService.prototype.refreshToken = /**
     * Attempt to refresh token
     * @return {?}
     */
    function () {
        var _this = this;
        return this.httpService
            .search({
            section: 'auth',
            cmd: 'refresh',
            rawResponse: true,
            insecure: true
        })
            .then((/**
         * @param {?} response
         * @return {?}
         */
        function (response) { return __awaiter(_this, void 0, void 0, function () {
            var newToken, claims;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!response.result) return [3 /*break*/, 3];
                        newToken = response.data.headers.get('authorization').split(' ')[1];
                        claims = this.jwtHelper.decodeToken(newToken);
                        return [4 /*yield*/, this.setUser(claims.user)];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.setToken(newToken)];
                    case 2:
                        _a.sent();
                        return [2 /*return*/, newToken];
                    case 3: return [2 /*return*/, ''];
                }
            });
        }); }));
    };
    /**
     * Clear all session variables
     */
    /**
     * Clear all session variables
     * @return {?}
     */
    AuthService.prototype.logout = /**
     * Clear all session variables
     * @return {?}
     */
    function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this._user = null;
                        return [4 /*yield*/, this.storageService.removeItem('jwt')];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.storageService.removeItem('user')];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    AuthService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    AuthService.ctorParameters = function () { return [
        { type: HttpService },
        { type: Router },
        { type: NgForage }
    ]; };
    return AuthService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var ApiLibModule = /** @class */ (function () {
    function ApiLibModule(parentModule) {
        if (parentModule) {
            throw new Error('ApiModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * @param {?} config
     * @return {?}
     */
    ApiLibModule.forRoot = /**
     * @param {?} config
     * @return {?}
     */
    function (config) {
        return {
            ngModule: ApiLibModule,
            providers: [
                { provide: EnvironmentConfigService, useValue: config },
                { provide: HttpService, useClass: 'ApiHttpService' },
                { provide: AuthService, useClass: 'ApiAuthService' },
            ]
        };
    };
    ApiLibModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [],
                    imports: [
                        CommonModule
                    ],
                    exports: []
                },] }
    ];
    /** @nocollapse */
    ApiLibModule.ctorParameters = function () { return [
        { type: ApiLibModule, decorators: [{ type: Optional }, { type: SkipSelf }] }
    ]; };
    return ApiLibModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { ApiLibModule, AuthService as ɵc, EnvironmentConfigService as ɵa, HttpService as ɵb };

//# sourceMappingURL=api-lib.js.map